#!/usr/bin/perl

=head1 create-image.pl

Packages an existing system up as an image

This command performs the complex process of converting an existing virtual
system into a Cloudmin image. It has three required parameters : C<--host>
followed by the hostname of the system to image, C<--desc> followed by a
description of the image, and C<--id> followed by a new unique image ID. The
ID is typically formatted like I<type-os-software>, such as
I<xen-centos6-virtualmin>.

If the ID refers to an image that already exists and you want to replace it,
add the C<--overwrite> paramter to the command line. You should also use
C<--version> followed by a version number, to make it easier to keep track of
different instances of the same image.

By default, files created as part of the image will be copied to the default
storage locations. Typically this is just the /var/webmin/server-manager
directory on the Cloudmin master. However, you can use the C<--storage>
flag followed by a storage spec to use a different location. This is typically
given in host:path format.

Creation takes several minutes, but the command will display each step in the
process as it proceeds, and inform you of its final success or failure. 

=head1 Example usage

  cloudmin create-image --host xenrails.home --desc "Image with rails" --id xen-centos5.0-rails
  Creating temporary directory for image files ..
  .. done
  Shutting down system ..
  .. done
  Mounting Xen instance filesystem ..
  .. mounted on /mnt/xen-xenrails
  Removing Virtualmin licence ..
  .. done
  Resetting root password ..
  .. done
  Removing authorized SSH keys ..
  .. done
  Un-mounting Xen instance filesystem ..
  .. done
  Compressing filesystem image ..
  .. done
  Transferring image files to Cloudmin master ..
  .. transferred all files
  Deleting compressed filesystem image ..
  .. done
  Mounting Xen instance filesystem ..
  .. mounted on /mnt/xen-xenrails
  Undoing licence and password changes ..
  .. done
  Un-mounting Xen instance filesystem ..
  .. done
  Starting up system ..
  .. done
  Refreshing system status ..
  .. done

=cut 

package server_manager;
$main::no_acl_check++;
$ENV{'WEBMIN_CONFIG'} ||= "/etc/webmin";
$ENV{'WEBMIN_VAR'} ||= "/var/webmin";
if ($0 =~ /^(.*)\/[^\/]+$/) {
	chdir($pwd = $1);
	}
else {
	chop($pwd = `pwd`);
	}
$0 = "$pwd/create-image.pl";
require './server-manager-lib.pl';
$< == 0 || die "$0 must be run as root";
&set_all_text_print();
@OLDARGV = @ARGV;
&can_action_usage(undef, "global");
&usage_if_cloudmin_replica();

# Parse command line
$munge = 1;
while(@ARGV > 0) {
	my $a = shift(@ARGV);
	if ($a eq "--host") {
		$host = shift(@ARGV);
		}
	elsif ($a eq "--desc") {
		$desc = shift(@ARGV);
		}
	elsif ($a eq "--id") {
		$id = shift(@ARGV);
		}
	elsif ($a eq "--overwrite") {
		$overwrite = 1;
		}
	elsif ($a eq "--version") {
		$version = shift(@ARGV);
		}
	elsif ($a eq "--inc-version") {
		$incversion = 1;
		}
	elsif ($a eq "--no-serial") {
		$munge = 0;
		}
	elsif ($a eq "--no-compress") {
		$nocompress = 1;
		}
	elsif ($a eq "--storage") {
		$dest = shift(@ARGV);
		}
	else {
		&usage();
		}
	}

# Check for mandatory args
$host || &usage("Missing --host parameter");
$id || &usage("Missing --id parameter");

# Get the server to image
$server = &get_managed_server_by("host", $host);
$server || &usage("No system with hostname $host was found");
$nfunc = "type_".$server->{'manager'}."_no_image";
if (defined(&$nfunc) && &$nfunc($server)) {
	&usage("The selected system type cannot be converted to an image");
	}
if ($server->{'status'} eq 'nohost') {
	&usage("A system whose host system is down cannot be imaged");
	}
($clash) = grep { $_->{'id'} eq $id } &list_server_images();
if ($clash) {
	$desc ||= $clash->{'desc'};
	if ($incversion && $clash->{'version'}) {
		$version = $clash->{'version'} + 0.1;
		}
	}
if (!$overwrite) {
	$clash && &usage("An image with the ID $id already exists, and --overwrite was not set");
	if (!$config{'vm2_overwrite'}) {
		$clash->{'vm2'} && &usage("The image with ID $id is provided ".
				  "by Virtualmin, and cannot be overwritten");
		}
	}
$desc || &usage("Missing --desc parameter");
&obtain_lock_server($server);

# Create initial image hash
$image = { 'id' => $id,
	   'desc' => $desc,
	   'manager' => $server->{'manager'},
	   'version' => $version,
	   'nocompress' => $nocompress,
	   'type' => $server->{'type'},
	 };
$info = &get_server_collected($server);
if ($info && $info->{'minfo'} && $info->{'minfo'}->{'version'}) {
	$image->{'virtualmin'} =
		$info->{'minfo'}->{'version'} =~ /gpl/i ||
		$info->{'minfo'}->{'virtualmin'} eq 'gpl' ? 'gpl' : 'pro';
	}
if ($info && $info->{'kernel'}) {
	$image->{'arch'} = $info->{'kernel'}->{'arch'};
	}

# Select temp dir for image on parent host
&$first_print("Creating temporary directory for image files ..");
$pfunc = "type_".$server->{'manager'}."_parent";
$vhost = &$pfunc($server);
$image->{'vhost'} = $vhost->{'id'};
if ($dest) {
	if (!&parse_backup_destination($dest, $vhost)) {
		&$second_print(".. storage location $dest is not valid");
		exit(3);
		}
	$image->{'dest'} = $dest;
	}
else {
	$err = &set_image_location($image, $vhost);
	if ($err) {
		&$second_print(".. no suitable image locations found : $err");
		exit(3);
		}
	}
($tempok, $temp) = &create_image_files_tempdir($vhost, $image->{'id'});
if (!$tempok) {
	&$second_print(".. failed : $temp");
	exit(3);
	}
else {
	&$second_print(".. done");
	}

$sfunc = "type_".$server->{'manager'}."_no_shutdown";
$noshut = defined(&$sfunc) && &$sfunc($server);
if ($server->{'status'} ne 'down' && !$noshut) {
	# Shut it down
	&$first_print("Shutting down system ..");
	$err = &shutdown_server($server);
	if ($err) {
		&$second_print(".. shutdown failed : $err");
		exit(2);
		}
	else {
		&$second_print(".. done");
		}
	$server->{'status'} = 'down';
	&save_managed_server($server);
	$shutdown = 1;
	}

# Validate first disk, if any
$dfunc = "type_".$server->{'manager'}."_list_disks";
$cffunc = "type_".$server->{'manager'}."_can_fsck";
$ffunc = "type_".$server->{'manager'}."_fsck_disk";
if (defined(&$dfunc) && defined(&$cffunc)) {
	&$first_print("Checking filesystem on first disk ..");
	$disks = &$dfunc($server, 1);
	if (!ref($disks)) {
		&$second_print(".. failed to list disks : $disks");
		}
	elsif (!@$disks) {
		&$second_print(".. no disks found!");
		}
	elsif (!&$cffunc($server, $disks->[0])) {
		&$second_print(".. not possible for the first disk");
		}
	else {
		($ok, $out) = &$ffunc($server, $disks->[0]);
		if ($ok == 2) {
			&$second_print(".. filesystem errors corrected");
			}
		elsif ($ok == 1) {
			&$second_print(".. filesystem is OK");
			}
		else {
			&$block_print($out);
			&$second_print(".. filesystem errors found!");
			exit(3);
			}
		}
	}

# Convert the remote system into an image file - this will output progress
$ifunc = "type_".$server->{'manager'}."_create_image";
if (!&$ifunc($server, $image, $temp, $munge ? 0 : 2)) {
	exit(3);
	}

# Transfer the image file to the master
$dstr = &nice_image_destination($image->{'dest'});
&$first_print("Transferring image files to $dstr ..");
$err = &fetch_image_files($vhost, $image, $temp, 0, $server);
if ($err) {
	&$second_print(".. failed : $err");
	}
else {
	&$second_print(".. transferred all files");

	# Save the new image file
	&generate_image_md5s($image);
	&generate_image_size($image);
	&save_server_image($image);
	}

# Cleanup the image on the host
$ifunc = "type_".$server->{'manager'}."_cleanup_image";
if (!&$ifunc($server, $image, $temp)) {
	exit(3);
	}

if ($shutdown) {
	&$first_print("Starting up system ..");
	($ok, $err) = &startup_server($server);
	if (!$ok) {
		&$second_print(".. startup failed : $err");
		exit(4);
		}
	else {
		&$second_print(".. done");
		}

	# Re-check status
	&$first_print("Refreshing system status ..");
	&check_server_status($server);
	&save_managed_server($server);
	&$second_print(".. done");
	}
&release_lock_server($server);

# Exit if image transfer failed, but only after cleanup is done
if ($err) {
	exit(5);
	}
else {
	&cloudmin_api_log(\@OLDARGV);
	}

sub usage
{
print "$_[0]\n\n" if ($_[0]);
print "Converts an existing virtual system into an image.\n";
print "\n";
print "cloudmin create-image --host name\n";
print "                      --desc \"image description\"\n";
print "                      --id image-id\n";
print "                     [--overwrite]\n";
print "                     [--version number | --inc-version]\n";
print "                     [--no-serial]\n";
print "                     [--no-compress]\n";
print "                     [--storage spec]\n";
exit(1);
}

